--------------------------------------------------
-- A Picture from One Newton Fractal
-- The Entry for The Programming Competition 2009
-- Name: Tomas Tauber
-- Matriculation number: 0943263
-- To Run: follow instructions in HOWTO
--------------------------------------------------
module Main where
import Plot
import Graphics.GD
import Data.Complex

imageDim :: Size
imageDim = (1000,1000)

-- How many iterations before we declare the point doesn't diverge
maxIter = 6144

-- How close successive tests must be before we declare they have converged
minDiff = 0.000001

-- formula to iterate
f z = z^8 + 15 * z^4 - 16
-- derivative
f' z = 8 * (z^7) + 60 * (z^3)

-- Coordinates in the complex plane to graph
-- First coordinate must be top left.
mbWindow :: Window
mbWindow = ( (-2.0, 2.0), (2.0, -2.0) ) -- Full view

main :: IO ()
main = drawPlot drawNewton imageDim mbWindow "picture.png"

drawNewton :: Coordinate -> Color
drawNewton (x,y) = colorIterations $ newton (x :+ y) (0,0)

-- Count number of iterations for the given point to diverge
-- Non-diveging points return (0,0)
newton :: Complex Double -- Coordinate to test
       -> (Int,Int) -- (Current iteration,a dummy variable)
       -> (Int,Int) -- (Iterations before diverging,a distinguished case)
newton z (iter,conv)
    | iter > maxIter = (0,0)
    | ( abs $ magnitude r - magnitude z) > minDiff = newton r ((iter + 1),0)
    | otherwise = decide r
    where r = z - f z / f' z
	  decide (rlp :+ imp) | ((rlp>imp)&&(rlp>0)) = (iter,1) --matches 2 roots> 1+0i and sqrt(2)-sqrt(2)i
			      | ((rlp<imp)&&(rlp<0)) = (iter,2) --matches 2 roots> -1+0i and -sqrt(2)+sqrt(2)i
			      | ((rlp<imp)&&(imp>0)) = (iter,3) --matches 1 of 2 roots (-sqrt(2)+sqrt(2) is evaluated as 2 because of the order)> 0+i
			      | ((rlp>imp)&&(imp<0)) = (iter,4) --matches 1 of 2 roots (sqrt(2)-sqrt(2) is evaluated as 1 because of the order)> 0-i
			      | otherwise = (iter,0) --ignoring the case (rlp==imp) which would match 2 other possible roots> -sqrt(2)-sqrt(2)i and sqrt(2)+sqrt(2)i

colorIterations :: (Int,Int) -> Color
colorIterations (x,y)
    | x > maxIter = rgb 255 255 255
    | y==1 = rgb (x*3) (x*3) x --brownish yellow (gold) dominates
    | y==2 = rgb x (x*3) x --green dominates
    | y==3 = rgb (x*3) x x --red dominates
    | y==4 = rgb x x (x*3) --blue dominates
    | otherwise = rgb x x x
